@@ -8,6 +8,8 @@
 #' value specifying label to use.
 #' @param values a named list (names are data colnames), each element is a single character
 #' value specifying label to use. If provided, argument `...` will be ignored.
+#' It can also be a unamed character vector, in that case, it must have the same
+#' length than the number of columns of the flextable.
 #' @examples
 #' ft <- flextable(head(iris))
 #' ft <- set_header_labels(ft,
@@ -28,9 +30,6 @@
 #' ft
 #' @export
 #' @family functions to add rows in a flextable
-#' @section Illustrations:
-#'
-#' \if{html}{\figure{fig_set_header_labels_1.png}{options: width="400"}}
 set_header_labels <- function(x, ..., values = NULL) {
   if (!inherits(x, "flextable")) {
     stop(sprintf("Function `%s` supports only flextable objects.", "set_header_labels()"))
@@ -42,16 +41,24 @@ set_header_labels <- function(x, ..., values = NULL) {
       stop("there is no header row to be replaced")
     }
   }
-
-  values <- values[names(values) %in% names(x$header$dataset)]
-  x$header$content[nrow_part(x, "header"), names(values)] <- as_paragraph(as_chunk(unlist(values)))
+  names_ <- names(values)
+  if (is.null(names_)) {
+    if (length(values) != ncol_keys(x)) {
+      stop("if unamed, the labels must have the same length than col_keys.")
+    }
+    x$header$content[nrow_part(x, "header"), ] <- as_paragraph(as_chunk(values))
+  } else {
+    names_ <- names_[names_ %in% x$col_keys]
+    values <- values[names_]
+    x$header$content[nrow_part(x, "header"), names_] <- as_paragraph(as_chunk(unlist(values)))
+  }
 
   x
 }
 
 
 #' @export
-#' @title delete flextable part
+#' @title Delete flextable part
 #'
 #' @description indicate to not print a part of
 #' the flextable, i.e. an header, footer or the body.
@@ -62,9 +69,6 @@ set_header_labels <- function(x, ..., values = NULL) {
 #' ft <- flextable(head(iris))
 #' ft <- delete_part(x = ft, part = "header")
 #' ft
-#' @section Illustrations:
-#'
-#' \if{html}{\figure{fig_delete_part_1.png}{options: width="300"}}
 delete_part <- function(x, part = "header") {
   if (!inherits(x, "flextable")) {
     stop(sprintf("Function `%s` supports only flextable objects.", "delete_part()"))
@@ -98,6 +102,43 @@ as_new_data <- function(x, ..., values = NULL) {
   data.frame(as.list(args_), check.names = FALSE, stringsAsFactors = FALSE)
 }
 
+add_rows_to_tabpart <- function( x, rows, first = FALSE ){
+
+  data <- x$dataset
+  spans <- x$spans
+  ncol <- length(x$col_keys)
+  nrow <- nrow(rows)
+
+  x$styles$cells <- add_rows(x$styles$cells, nrows = nrow, first = first)
+  x$styles$pars <- add_rows(x$styles$pars, nrows = nrow, first = first)
+  x$styles$text <- add_rows(x$styles$text, nrows = nrow, first = first)
+  x$content <- add_rows(x$content, nrows = nrow, first = first, rows)
+
+  span_new <- matrix( 1, ncol = ncol, nrow = nrow )
+  rowheights <- x$rowheights
+  hrule <- x$hrule
+
+  if( !first ){
+    data <- rbind(data, rows )
+    spans$rows <- rbind( spans$rows, span_new )
+    spans$columns <- rbind( spans$columns, span_new )
+    rowheights <- c(rowheights, rep(rev(rowheights)[1], nrow(rows)))
+    hrule <- c(hrule, rep(rev(hrule)[1], nrow(rows)))
+  } else {
+    data <- rbind(rows, data )
+    spans$rows <- rbind( span_new, spans$rows )
+    spans$columns <- rbind( span_new, spans$columns )
+    rowheights <- c(rep(rowheights[1], nrow(rows)), rowheights)
+    hrule <- c(rep(hrule[1], nrow(rows)), hrule)
+
+  }
+  x$rowheights <- rowheights
+  x$dataset <- data
+  x$spans <- spans
+  x$hrule <- hrule
+  x
+}
+
 # add header/footer content ----
 
 #' @export
@@ -148,7 +189,7 @@ add_body <- function(x, top = TRUE, ..., values = NULL) {
   }
 
   new_data <- as_new_data(x = x, ..., values = values)
-  x$body <- add_rows(x$body, new_data, first = top)
+  x$body <- add_rows_to_tabpart(x$body, new_data, first = top)
   x
 }
 
@@ -204,7 +245,7 @@ add_header <- function(x, top = TRUE, ..., values = NULL) {
   }
 
   header_data <- as_new_data(x = x, ..., values = values)
-  x$header <- add_rows(x$header, header_data, first = top)
+  x$header <- add_rows_to_tabpart(x$header, header_data, first = top)
 
   x
 }
@@ -255,7 +296,7 @@ add_footer <- function(x, top = TRUE, ..., values = NULL) {
   if (nrow_part(x, "footer") < 1) {
     x$footer <- complex_tabpart(data = footer_data, col_keys = x$col_keys, cwidth = .75, cheight = .25)
   } else {
-    x$footer <- add_rows(x$footer, footer_data, first = top)
+    x$footer <- add_rows_to_tabpart(x$footer, footer_data, first = top)
   }
 
   x
@@ -372,7 +413,7 @@ add_body_row <- function(x, top = TRUE, values = list(), colwidths = integer(0))
       cwidth = dim(x)$widths, cheight = .25
     )
   } else {
-    x$body <- add_rows(x$body, body_data, first = top)
+    x$body <- add_rows_to_tabpart(x$body, body_data, first = top)
   }
 
   i <- ifelse(top, 1, nrow_part(x, "body"))
@@ -460,7 +501,7 @@ add_header_row <- function(x, top = TRUE, values = character(0), colwidths = int
   if (nrow_part(x, "header") < 1) {
     x$header <- complex_tabpart(data = header_data, col_keys = x$col_keys, cwidth = dim(x)$widths, cheight = .25)
   } else {
-    x$header <- add_rows(x$header, header_data, first = top)
+    x$header <- add_rows_to_tabpart(x$header, header_data, first = top)
   }
 
   row_span <- unlist(lapply(colwidths, function(x) {
@@ -562,7 +603,7 @@ add_footer_row <- function(x, top = TRUE, values = character(0), colwidths = int
   if (nrow_part(x, "footer") < 1) {
     x$footer <- complex_tabpart(data = footer_data, col_keys = x$col_keys, cwidth = dim(x)$widths, cheight = .25)
   } else {
-    x$footer <- add_rows(x$footer, footer_data, first = top)
+    x$footer <- add_rows_to_tabpart(x$footer, footer_data, first = top)
   }
 
   row_span <- unlist(lapply(colwidths, function(x) {
@@ -617,8 +658,6 @@ data_from_char <- function(values, colwidths, col_keys) {
 #' @param top should the row be inserted at the top
 #' or the bottom. Default to TRUE.
 #' @family functions that add rows in the table
-#' @section Illustrations:
-#' \if{html}{\figure{fig_add_header_lines_1.png}{options: width="300"}}
 #' @examples
 #' # ex 1----
 #' ft_1 <- flextable(head(iris))
@@ -679,8 +718,6 @@ add_header_lines <- function(x, values = character(0), top = TRUE) {
 #' add labels in the footer, a footnote for example.
 #' @inheritParams add_header_lines
 #' @family functions to add rows in a flextable
-#' @section Illustrations:
-#' \if{html}{\figure{fig_add_footer_lines_1.png}{options: width="300"}}
 #' @examples
 #' ft_1 <- flextable(head(iris))
 #' ft_1 <- add_footer_lines(ft_1,
@@ -797,11 +834,6 @@ set_part_df <- function(x, mapping = NULL, key = "col_keys", part) {
 #' ft_1 <- fix_border_issues(ft_1)
 #' ft_1
 #' @family functions to add rows in a flextable
-#' @section Illustrations:
-#'
-#' \if{html}{\figure{fig_set_header_footer_df_1.png}{options: width="400"}}
-#'
-#' \if{html}{\figure{fig_set_header_footer_df_2.png}{options: width="400"}}
 set_header_df <- function(x, mapping = NULL, key = "col_keys") {
   if (!inherits(x, "flextable")) {
     stop(sprintf("Function `%s` supports only flextable objects.", "set_header_df()"))
